=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2007-07-10 21:18:54 +0000
+++ bzrlib/branch.py	2007-07-11 14:58:54 +0000
@@ -1962,7 +1962,7 @@
 
     def _check_history_violation(self, revision_id):
         last_revision = self.last_revision()
-        if last_revision is None:
+        if _mod_revision.is_null(last_revision):
             return
         if last_revision not in self._lefthand_history(revision_id):
             raise errors.AppendRevisionsOnlyViolation(self.base)

=== modified file 'bzrlib/memorytree.py'
--- bzrlib/memorytree.py	2007-07-10 21:18:54 +0000
+++ bzrlib/memorytree.py	2007-07-11 14:58:54 +0000
@@ -231,6 +231,8 @@
 
     def set_parent_ids(self, revision_ids, allow_leftmost_as_ghost=False):
         """See MutableTree.set_parent_trees()."""
+        for revision_id in revision_ids:
+            _mod_revision.check_not_reserved_id(revision_id)
         if len(revision_ids) == 0:
             self._parent_ids = []
             self._basis_tree = self.branch.repository.revision_tree(None)
@@ -250,7 +252,8 @@
                 # a ghost in the left most parent
                 raise errors.GhostRevisionUnusableHere(parents_list[0][0])
             self._parent_ids = [parent_id for parent_id, tree in parents_list]
-            if parents_list[0][1] is None:
+            if parents_list[0][1] is None or parents_list[0][1] == 'null:':
+                import pdb; pdb.set_trace()
                 self._basis_tree = self.branch.repository.revision_tree(None)
             else:
                 self._basis_tree = parents_list[0][1]

=== modified file 'bzrlib/revision.py'
--- bzrlib/revision.py	2007-07-10 21:18:54 +0000
+++ bzrlib/revision.py	2007-07-11 14:58:54 +0000
@@ -483,6 +483,7 @@
     if is_reserved_id(revision_id):
         raise errors.ReservedId(revision_id)
 
+
 def ensure_null(revision_id):
     """Ensure only NULL_REVISION is used to represent the null revisionn"""
     if revision_id is None:
@@ -490,5 +491,6 @@
     else:
         return revision_id
 
+
 def is_null(revision_id):
     return revision_id in (None, NULL_REVISION)

=== modified file 'bzrlib/tests/branch_implementations/test_uncommit.py'
--- bzrlib/tests/branch_implementations/test_uncommit.py	2007-02-06 02:33:42 +0000
+++ bzrlib/tests/branch_implementations/test_uncommit.py	2007-07-11 14:58:54 +0000
@@ -105,4 +105,3 @@
              1, revid, None, True)
             ],
             self.hook_calls)
-

=== modified file 'bzrlib/tests/branch_implementations/test_update.py'
--- bzrlib/tests/branch_implementations/test_update.py	2006-10-11 23:08:27 +0000
+++ bzrlib/tests/branch_implementations/test_update.py	2007-07-11 14:58:54 +0000
@@ -15,7 +15,10 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
-import bzrlib.errors as errors
+from bzrlib import (
+    errors,
+    revision as _mod_revision,
+    )
 from bzrlib.tests.branch_implementations.test_branch import TestCaseWithBranch
 
 
@@ -27,7 +30,7 @@
     def test_update_unbound_works(self):
         b = self.make_branch('.')
         b.update()
-        self.assertEqual(None, b.last_revision())
+        self.assertEqual(_mod_revision.NULL_REVISION, b.last_revision())
 
     def test_update_prefix_returns_none(self):
         # update in a branch when its a prefix of the master should

=== modified file 'bzrlib/tests/bzrdir_implementations/test_bzrdir.py'
--- bzrlib/tests/bzrdir_implementations/test_bzrdir.py	2007-05-01 22:41:41 +0000
+++ bzrlib/tests/bzrdir_implementations/test_bzrdir.py	2007-07-11 14:58:54 +0000
@@ -28,6 +28,7 @@
     errors,
     lockdir,
     repository,
+    revision as _mod_revision,
     transactions,
     transport,
     ui,
@@ -586,7 +587,8 @@
         repo.fetch(tree.branch.repository)
         self.assertTrue(repo.has_revision('1'))
         try:
-            self.assertIs(dir.open_branch().last_revision(), None)
+            self.assertTrue(
+                _mod_revision.is_null(dir.open_branch().last_revision()))
         except errors.NotBranchError:
             pass
         target = self.sproutOrSkip(dir, self.get_url('target'))

=== modified file 'bzrlib/tests/workingtree_implementations/test_commit.py'
--- bzrlib/tests/workingtree_implementations/test_commit.py	2007-06-28 03:21:59 +0000
+++ bzrlib/tests/workingtree_implementations/test_commit.py	2007-07-11 14:58:54 +0000
@@ -18,7 +18,15 @@
 from cStringIO import StringIO
 import os
 
-from bzrlib import branch, bzrdir, errors, ui, workingtree
+from bzrlib import (
+    branch,
+    bzrdir,
+    errors,
+    revision as _mod_revision,
+    ui,
+    uncommit,
+    workingtree,
+    )
 from bzrlib.errors import (NotBranchError, NotVersionedError, 
                            UnsupportedOperation)
 from bzrlib.osutils import pathjoin, getcwd, has_symlinks
@@ -152,7 +160,7 @@
             return
         tree.commit('foo', rev_id='foo', local=True)
         self.failIf(master.repository.has_revision('foo'))
-        self.assertEqual(None, master.last_revision())
+        self.assertTrue(_mod_revision.is_null(master.last_revision()))
 
     def test_record_initial_ghost(self):
         """The working tree needs to record ghosts during commit."""
@@ -383,3 +391,14 @@
              ],
             factory._calls
            )
+
+
+class TestUncommit(TestCaseWithWorkingTree):
+
+    def test_uncommit_to_null(self):
+        tree = self.make_branch_and_tree('branch')
+        tree.lock_write()
+        revid = tree.commit('a revision')
+        tree.unlock()
+        uncommit.uncommit(tree.branch, tree=tree)
+        self.assertEqual([], tree.get_parent_ids())

=== modified file 'bzrlib/tests/workingtree_implementations/test_parents.py'
--- bzrlib/tests/workingtree_implementations/test_parents.py	2006-10-11 23:08:27 +0000
+++ bzrlib/tests/workingtree_implementations/test_parents.py	2007-07-11 14:58:54 +0000
@@ -18,7 +18,11 @@
 
 import os
 
-from bzrlib import errors, symbol_versioning
+from bzrlib import (
+    errors,
+    revision as _mod_revision,
+    symbol_versioning,
+    )
 from bzrlib.tests.workingtree_implementations import TestCaseWithWorkingTree
 from bzrlib.uncommit import uncommit
 
@@ -33,7 +37,7 @@
         """
         self.assertEqual(expected, tree.get_parent_ids())
         if expected == []:
-            self.assertEqual(None, tree.last_revision())
+            self.assertTrue(_mod_revision.is_null(tree.last_revision()))
         else:
             self.assertEqual(expected[0], tree.last_revision())
         self.assertEqual(expected[1:],
@@ -41,6 +45,13 @@
                 tree.pending_merges))
 
 
+class TestGetParents(TestParents):
+
+    def test_get_parents(self):
+        t = self.make_branch_and_tree('.')
+        self.assertEqual([], t.get_parent_ids())
+
+
 class TestSetParents(TestParents):
 
     def test_set_no_parents(self):
@@ -52,6 +63,13 @@
         t.set_parent_trees([])
         self.assertConsistentParents([], t)
 
+    def test_set_null_parent(self):
+        t = self.make_branch_and_tree('.')
+        self.assertRaises(errors.ReservedId, t.set_parent_ids, ['null:'],
+                          allow_leftmost_as_ghost=True)
+        self.assertRaises(errors.ReservedId, t.set_parent_trees,
+                          [('null:', None)], allow_leftmost_as_ghost=True)
+
     def test_set_one_ghost_parent_rejects(self):
         t = self.make_branch_and_tree('.')
         self.assertRaises(errors.GhostRevisionUnusableHere,

=== modified file 'bzrlib/uncommit.py'
--- bzrlib/uncommit.py	2007-02-06 02:33:42 +0000
+++ bzrlib/uncommit.py	2007-07-11 14:58:54 +0000
@@ -20,6 +20,7 @@
 
 import os
 
+from bzrlib import revision as _mod_revision
 from bzrlib.branch import Branch
 from bzrlib.errors import BoundBranchOutOfDate
 
@@ -84,10 +85,13 @@
                 hook_local = branch
                 hook_master = master
             for hook in Branch.hooks['post_uncommit']:
+                hook_new_tip = new_tip
+                if hook_new_tip == _mod_revision.NULL_REVISION:
+                    hook_new_tip = None
                 hook(hook_local, hook_master, old_revno, old_tip, new_revno,
-                    new_tip)
+                     hook_new_tip)
             if tree is not None:
-                if new_tip is not None:
+                if not _mod_revision.is_null(new_tip):
                     parents = [new_tip]
                 else:
                     parents = []

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2007-07-10 21:18:54 +0000
+++ bzrlib/workingtree.py	2007-07-11 14:58:54 +0000
@@ -504,7 +504,7 @@
         value and uses that to decide what the parents list should be.
         """
         last_rev = self._last_revision()
-        if last_rev is None:
+        if _mod_revision.is_null(last_rev):
             parents = []
         else:
             parents = [last_rev]
@@ -735,6 +735,8 @@
         revision_ids = [osutils.safe_revision_id(r) for r in revision_ids]
         self._check_parents_for_ghosts(revision_ids,
             allow_leftmost_as_ghost=allow_leftmost_as_ghost)
+        for revision_id in revision_ids:
+            _mod_revision.check_not_reserved_id(revision_id)
 
         if len(revision_ids) > 0:
             self.set_last_revision(revision_ids[0])
@@ -747,6 +749,8 @@
     def set_parent_trees(self, parents_list, allow_leftmost_as_ghost=False):
         """See MutableTree.set_parent_trees."""
         parent_ids = [osutils.safe_revision_id(rev) for (rev, tree) in parents_list]
+        for revision_id in parent_ids:
+            _mod_revision.check_not_reserved_id(revision_id)
 
         self._check_parents_for_ghosts(parent_ids,
             allow_leftmost_as_ghost=allow_leftmost_as_ghost)
@@ -812,7 +816,7 @@
             else:
                 to_revision = osutils.safe_revision_id(to_revision)
             merger.other_rev_id = to_revision
-            if merger.other_rev_id is None:
+            if _mod_revision.is_null(merger.other_rev_id):
                 raise errors.NoCommits(branch)
             self.branch.fetch(branch, last_revision=merger.other_rev_id)
             merger.other_basis = merger.other_rev_id
@@ -2619,7 +2623,11 @@
         if basis_tree.inventory.root is not None:
             wt.set_root_id(basis_tree.inventory.root.file_id)
         # set the parent list and cache the basis tree.
-        wt.set_parent_trees([(revision, basis_tree)])
+        if _mod_revision.is_null(revision):
+            parent_trees = []
+        else:
+            parent_trees = [(revision, basis_tree)]
+        wt.set_parent_trees(parent_trees)
         transform.build_tree(basis_tree, wt)
         return wt
 

=== modified file 'bzrlib/workingtree_4.py'
--- bzrlib/workingtree_4.py	2007-06-06 06:11:58 +0000
+++ bzrlib/workingtree_4.py	2007-07-11 14:58:54 +0000
@@ -52,6 +52,7 @@
     ignores,
     merge,
     osutils,
+    revision as _mod_revision,
     revisiontree,
     textui,
     transform,
@@ -1089,6 +1090,7 @@
         # missing on access.
         for rev_id, tree in parents_list:
             rev_id = osutils.safe_revision_id(rev_id)
+            _mod_revision.check_not_reserved_id(rev_id)
             if tree is not None:
                 real_trees.append((rev_id, tree))
             else:

=== modified file 'bzrlib/xml5.py'
--- bzrlib/xml5.py	2007-03-02 08:55:16 +0000
+++ bzrlib/xml5.py	2007-07-11 14:58:54 +0000
@@ -21,6 +21,7 @@
     cache_utf8,
     errors,
     inventory,
+    revision as _mod_revision,
     )
 from bzrlib.xml_serializer import SubElement, Element, Serializer
 from bzrlib.inventory import ROOT_ID, Inventory, InventoryEntry
@@ -256,6 +257,7 @@
             pelts.tail = pelts.text = '\n'
             for parent_id in rev.parent_ids:
                 assert isinstance(parent_id, basestring)
+                _mod_revision.check_not_reserved_id(parent_id)
                 p = SubElement(pelts, 'revision_ref')
                 p.tail = '\n'
                 if isinstance(parent_id, str):

